#!/bin/bash

if [[ "$#" != "3" ]]; then
    echo "Usage: test_mempressure.sh <fs_mb> <dirty_mb> <crypted>"
    exit 1
fi

backing_size="$1"
crud_to_write="$2"
crypted="$3"

if [[ "$crypted" != "0" ]] && [[ "$crypted" != "1" ]]; then
    echo "crypted must be 0 or 1"
    exit 1
fi

die() {
    echo "Dying:" "$@"
    exit 1
}

check() {
    "$@" || die Failed to "$@"
}

mkdir mountpoint
[[ -d mountpoint ]] || die Failed to create mountpoint

mkdir ramfs_mountpoint
[[ -d ramfs_mountpoint ]] || die Failed to create ramfs_mountpoint

check mount -t ramfs none ramfs_mountpoint
trap "umount ramfs_mountpoint" EXIT

echo "Writing ${backing_size}M of zeros to ramfs..."
check dd if=/dev/zero of=ramfs_mountpoint/backing bs=1M count="$backing_size"

loopdev=`losetup -f --show ramfs_mountpoint/backing` || die losetup failed
echo "Using loopback device $loopdev."
trap "umount ramfs_mountpoint; losetup -d $loopdev" EXIT

if [[ "$crypted" == "1" ]]; then
    echo "Setting up dm-crypt device crypted_backing..."
    check echo "password" |cryptsetup create -s 256 -c aes-xts-plain -d- crypted_backing "$loopdev"
    backing=/dev/mapper/crypted_backing
    teardown() {
	echo "Cleaning up..."
	umount mountpoint
	cryptsetup remove crypted_backing
	losetup -d "$loopdev"
	umount ramfs_mountpoint
    }
    trap teardown EXIT
else
    backing="$loopdev"
    teardown() {
	echo "Cleaning up..."
	umount mountpoint
	losetup -d "$loopdev"
	umount ramfs_mountpoint
    }
    trap teardown EXIT
fi
echo "Backing device is $loopdev."

echo "Creating ext4 filesystem..."
check mkfs -t ext4 -q "$backing"

echo "Mounting..."
check mount -t ext4 "$backing" mountpoint

echo "Syncing..."
sync

echo "Here goes...  (hit Ctrl-C when you get bored)"

while true; do
    echo "Dirtying ${crud_to_write}M of memory-backed pages..."
    check dd if=/dev/zero of=mountpoint/garbage bs=1M count="$crud_to_write"

    echo "Waiting one second.."
    sleep 1

    echo "Freeing..."
    check rm mountpoint/garbage
done
